/*
*  This script defines functions used to show and to remove duplicates within ManageDuplicates.xul
*
* Author: Stphane BERTIN
* http://bookmarkdd.mozdev.org
*
*/

var URLnum = 0;
var DupNum = 0;
var Totalnum = 0;
var AutoDeletedBookmarkSameFolder= 0;
var AutoDeletedBookmarkSelectedFolder= 0;
var URLParents = new Array();
var URLrdfNodes = new Array();
var SelectedURL = "";
var rssFeed = /(.rss)$/i;	//Verify if url is a FeedURL
var isNotRootFolder = /( \> )/;		//Search if there is ' > ' if not it is root folder.

var DuplicatesBundle = document.getElementById('duplicates-strings');
var URLlist = document.getElementById("URLlist");
var FolderslistJS = document.getElementById("Folderslist");

var AutoDeleteSameFolder = UserPrefs.getPref("bool","AutoDelete.BookmarkInSameFolder");
var AutoDeleteSelectedFolder = UserPrefs.getPref("bool","AutoDelete.BookmarkInSelectedFolder");

const TRUNCATENUM = UserPrefs.getPref("int","URL.truncateNum");


function onLoad(WinArgs) {
    var messageAutoDelete = "";
    DupNum = new String(WinArgs[1]);
    Totalnum = new String(WinArgs[2]);
    setMessage(DupNum, Totalnum);


    if("arguments" in window && WinArgs.length > 0) {
	var listURL = WinArgs[0];
    }

    for(var i=0;i<listURL.length;i++){
	if(AutoDeleteSameFolder || AutoDeleteSelectedFolder){
	    if (! AutoDeleteDuplicatesBookmark(listURL[i])) addURL(URLlist,listURL[i],i);
	} else addURL(URLlist,listURL[i],i);
    }

    if (AutoDeletedBookmarkSameFolder>0 ){
	messageAutoDelete = DuplicatesBundle.getFormattedString("DuplicatesAutoDelete-SameFolder", [AutoDeletedBookmarkSameFolder]);
	alert(messageAutoDelete);
    }
    if (AutoDeletedBookmarkSelectedFolder>0 ){
	messageAutoDelete = DuplicatesBundle.getFormattedString("DuplicatesAutoDelete-SelectedFolder", [AutoDeletedBookmarkSelectedFolder]);
	alert(messageAutoDelete);
    }

    if (listURL.length>15){URLlist.setAttribute('rows','15');}

    URLlist.setAttribute('onclick','ShowFolders(event,this.selectedItem);');
    URLlist.setAttribute('onkeypress','ShowFolders(event,this.selectedItem);');

    window.sizeToContent();

    window.addEventListener("keypress", searchEscape, true);

}

// Event Listener to detect escape. Escape close window only if no item is selected.
// If a folder is selected it needs three escape key press to close window.
function searchEscape(aEvent){
    //Bookmarkdd.myDump("Key pressed is: "+aEvent.keyCode);

    if (aEvent.keyCode && aEvent.keyCode == "27"){
	if (FolderslistJS.selectedCount != 0) {
	    FolderslistJS.clearSelection();
	    return;
	}

	if (URLlist.selectedCount != 0) {
	    document.getElementById("description-url").setAttribute('value',"");
	    removeAllItems("Folderslist");
	    URLlist.clearSelection();
	    return;
	}

	window.close();
    }
}

function addURL(list,value,index) {
    list.appendItem(value,index);
}

function setMessage(Num, Total){
    var message ="";
    try {
	message = DuplicatesBundle.getFormattedString("DuplicatesFound", [Num, Total]);
    }  catch(e){
    	message = "You are using a localisation which is not supported !";
    }

    var messageParent = document.getElementById("Find-message");
    messageParent.setAttribute('value',message);
}


function ShowFolders(aEvent,aItem){
    //if(aEvent.button !=0) return false; //Why restrict use of button 0 ??
    var folders = null;

    removeAllItems("Folderslist");

    // Each time an item is selected 'isBookmarked' is executed.
    try {
	var urlFeed = rssFeed.exec(aItem.label);
	if (!urlFeed) {
	    folders = Bookmarkdd.isBookmarked(aItem.label);
	} else {
	    folders = Bookmarkdd.isBookmarked(aItem.label,aItem.label);
	}
    } catch(e) { return false; }

    SelectedURL = aItem;

    if(folders.parents) {
	URLParents = folders.parents;
	URLrdfNodes = folders.RDFnodes;
	URLnum = folders.parents.length;
    } else {
	Bookmarkdd.myDump("ShowFolders: impossible to find duplicate URLs !");
	return false;
    }
    if(URLnum > 1)  document.getElementById('b_delete').removeAttribute('disabled');
    if(URLnum == 2)  {
	FolderslistJS.setAttribute('seltype','single');
    } else FolderslistJS.setAttribute('seltype','multiple');

    for(var i=0;i<URLnum;i++){ addURL(FolderslistJS,URLParents[i],i); }

    document.getElementById("Folderslist").setAttribute('rows',eval(URLnum+1));
    var TruncatedUrl = aItem.label.substr(0,TRUNCATENUM);
    document.getElementById("description-url").setAttribute('value',TruncatedUrl);

    var WindowSizeMode = document.getElementById("bookmarkDD-ManageDuplicates").getAttribute('sizemode');
    //alert("Window Sizemode= "+WindowSizeMode);
    if(WindowSizeMode != "maximized"){
	window.sizeToContent();
    }

    return true;
}


function removeAllItems(myListBox) {

    var box = document.getElementById(myListBox);
    var rowCount = box.getRowCount();
    for( var i = rowCount; i > 0; i-- ) {
	box.removeItemAt(0);
    }

}


function deleteSelection() {
    var count = FolderslistJS.selectedCount;
    //Bookmarkdd.myDump("Number of folders selected: "+count);
    count--;
    for(var i=count;i>=0;i--){
	if(!removeURL(FolderslistJS.selectedItems[i])) continue;
	FolderslistJS.removeItemAt(FolderslistJS.getIndexOfItem(FolderslistJS.selectedItems[i]));

	if(URLnum == 2)  {
	    FolderslistJS.setAttribute('seltype','single');
	} else FolderslistJS.setAttribute('seltype','multiple');

	//If there is only one folder disable delete button to avoid remove of all URL references
	if (FolderslistJS.getRowCount()<2) {
	    document.getElementById('b_delete').setAttribute('disabled','true');
	    URLlist.removeItemAt(URLlist.getIndexOfItem(SelectedURL));

	    removeAllItems("Folderslist");
	    document.getElementById("description-url").setAttribute('value',"");

	    DupNum--;
	    setMessage(DupNum, Totalnum);
	    return true;
	}
    }
    return true;
}

function removeURL(aItem){

    var URLPath = URLParents[aItem.value];
    var TruncatedUrl = SelectedURL.label.substr(0,TRUNCATENUM);
    var message = DuplicatesBundle.getFormattedString("Remove-Confirmation", [TruncatedUrl, URLPath]);

    var result = true;
    //If confirmation is not asked delete URL
    if(UserPrefs.getPref("bool","removeURL.Askconfirmation")) result = BddConfirm(message);
    if (result) {
	var selection = Bookmarkdd.createSelection(URLrdfNodes[aItem.value]);
	//Remove bookmark in Bookmarks
	//XXX Stiffux: Comment following line to DEBUG
	BookmarksCommand.deleteBookmark(selection);
	URLnum--;
	return true;
    }
    return false;
}



function openbmProperties(aItem,aEvent){
    //Show Properties if context Click button --> rigth mouse click
    if (aEvent.button != 2) return false;
    var value = {};
    window.openDialog("chrome://browser/content/bookmarks/bookmarksProperties.xul", "bmProperties", "centerscreen,chrome,modal,resizable", URLrdfNodes[aItem.value].Value, value);
    return value.ok;
}


// If two duplicates bookmarks are in same folder delete the second found.
// OR if a duplicate bookmark is in selected folder delete it.
function AutoDeleteDuplicatesBookmark(aURL){
    var folders = null;
    var URLPath = new Array();
    var selectedFolder = UserPrefs.getPref("string","AutoDelete.SelectedFolder");
    var searchSelectedFolder = new RegExp("^("+selectedFolder+")","i");

    //WARNING message: selection of root Folder 'Bookmarks' will delete ONLY duplicates which aren't in subfolders.
    if (!isNotRootFolder.exec(selectedFolder)) {
    	//Bookmarkdd.myDump("It is the root Folder: "+selectedFolder);
	searchSelectedFolder = new RegExp("^("+selectedFolder+")$","i");
    }

    var urlFeed = rssFeed.exec(aURL);
    if (!urlFeed) {
	folders = Bookmarkdd.isBookmarked(aURL);
    } else {
	folders = Bookmarkdd.isBookmarked(aURL,aURL);
    }

    var num = 0;
    if(folders.parents) {
	var URLParents = folders.parents;
	var URLrdfNodes = folders.RDFnodes;
	num = folders.parents.length;
    } else {
	Bookmarkdd.myDump("AutoDeleteDuplicatesBookmark: impossible to find duplicate URLs for:"+ aURL);
	return false;
    }
    
    for (var i=0;i<folders.parents.length; i++){
	var alreadyIn = false;
	if(Bookmarkdd.version == "1.5"){
		if(URLPath.indexOf(URLParents[i]) != -1) alreadyIn = true;
	} else {
		for (var count = 0; count < URLPath.length;count++){
			if(!URLPath[count]) continue;
			if(URLPath[count] == URLParents[i]) { alreadyIn = true; break; }
		}
	}

    	if(AutoDeleteSameFolder && alreadyIn) {
	    //Remove bookmark in Bookmarks
	    //Bookmarkdd.myDump("URL: "+aURL+" duplicate in same folder: "+URLParents[i]);
	    var selection = Bookmarkdd.createSelection(URLrdfNodes[i]);
	    BookmarksCommand.deleteBookmark(selection);
	    AutoDeletedBookmarkSameFolder++;
	    num--;
	} else if(AutoDeleteSelectedFolder && searchSelectedFolder.exec(URLParents[i])) {
	    //Remove bookmark in Bookmarks
	    //Bookmarkdd.myDump("URL: "+aURL+" duplicate in selected folder: "+URLParents[i]);
	    var selection = Bookmarkdd.createSelection(URLrdfNodes[i]);
	    BookmarksCommand.deleteBookmark(selection);
	    AutoDeletedBookmarkSelectedFolder++;
	    num--;
	} else {
	    URLPath.push(URLParents[i]);
	}
    }

    // If there is no more duplicate of this aURL, delete it in URLlist
    if( num == 1 ){
	//URLlist.removeItemAt(URLlist.getIndexOfItem(aURL));
	DupNum--;
	setMessage(DupNum, Totalnum);
	return true;
    }

    return false;
}
